
Object subclass: #SpeedMouse
  instanceVariableNames: 
    ' time oldTime position oldPosition velocity oldVelocity acceleration significantDeltaPosition significantDeltaTime '
  classVariableNames: ''
  poolDictionaries: ''
  category: 'JOOP4'
!

TextWindow subclass: #SpeedReadingWindow
  instanceVariableNames: 
    ' speedMouse '
  classVariableNames: ''
  poolDictionaries: ''
  category: 'JOOP4'
!

TextPane subclass: #SpeedReadingTextPane
  instanceVariableNames: 
    ' speedMouse timer pixelsScrolled autoScrolling stream '
  classVariableNames: ''
  poolDictionaries: 
    ' OperatingSystemConstants '
  category: 'JOOP4'
!


!SpeedMouse class methodsFor: 'creating'!
   
new
    ^super new initialize! !


!SpeedMouse methodsFor: 'initializing'!
  
initialize
    | zero now |
    zero := 0.0.
    now := Time millisecondClockValue.
    self oldTime: now; time: now.
    self oldPosition: zero; position: zero.
    self oldVelocity: zero; velocity: zero.
    self acceleration: zero.
    self significantDeltaPosition: 10@10.
    self significantDeltaTime: 40.
! !

!SpeedMouse methodsFor: 'get/set'!
 
significantDeltaPosition
    ^significantDeltaPosition!

significantDeltaPosition: aPoint
    significantDeltaPosition := aPoint!
   
oldPosition: aPoint
    oldPosition := aPoint!
 
oldVelocity: aPoint
    oldVelocity := aPoint!
 
position
    ^position!

oldTime: aTime
    oldTime := aTime!
   
position: aPoint
    position := aPoint!
   
time
    ^time!

velocity: aPoint
    velocity := aPoint!
   
significantDeltaTime: aFloat
    significantDeltaTime := aFloat!
   
velocity
    ^velocity!

acceleration: aPoint
    acceleration := aPoint!
   
acceleration
    ^acceleration!

significantDeltaTime
    ^significantDeltaTime!

oldPosition
    ^oldPosition!
  
time: aTime
    time := aTime!
 
oldVelocity
    ^oldVelocity!
  
oldTime
    ^oldTime! !

!SpeedMouse methodsFor: 'querying'!
 
deltaPosition
    ^self position - self oldPosition!
   
deltaVelocity
    ^self velocity - self oldVelocity!
   
deltaTime
    ^(self time - self oldTime)" max: 1"! !

!SpeedMouse methodsFor: 'tracking'!
   
positionNow: aPoint
    | now |
    now := Time millisecondClockValue.
"times add: now. positions add: aPoint."
    self updateFromPosition: aPoint time: now.
    self revisionRequired ifTrue: [
        self pushTime; pushPosition; pushVelocity]!

revisionRequired
    | delta |
    delta := self deltaTime.
    delta >= self significantDeltaTime ifTrue: [^true].
    delta == 0 ifTrue: [^false].
    ^self deltaPosition abs >= self significantDeltaPosition!
 
updateFromPosition: aPoint time: aTime
    | duration |
    self time: aTime; position: aPoint.
    duration := self deltaTime asFloat.
    duration < 0.5 ifTrue: [^self].
    self
        velocity: self deltaPosition / duration;
        acceleration: self deltaVelocity / duration
! !

!SpeedMouse methodsFor: 'private'!
 
pushTime
    self oldTime: self time!
  
pushVelocity
    self oldVelocity: self velocity!
  
pushPosition
    self oldPosition: self position! !
SpeedMouse comment: ''!


!SpeedReadingWindow class methodsFor: 'examples'!

example1
    "SpeedReadingWindow example1 openFile"
    ^self new openOn: 'Speed window test...'.! !

!SpeedReadingWindow class methodsFor: 'overrides'!

textWindowClass
    ^self! !


!SpeedReadingWindow methodsFor: 'overrides'!
 
textPaneClass
    ^SpeedReadingTextPane!
   
openFile
    "Private - User selected Open... from the File menu."
    | dialog file |
    (dialog := FileDialog new openFile ) isNil
        ifTrue: [ ^self ].
    (file := dialog file ) isNil
        ifTrue: [ ^self ].
    (File exists: file ) ifFalse: [
        (MessageBox confirm: 'File does not exist.  Do you want to create it?' )
            ifTrue: [ File createFileNamed: file ]
            ifFalse: [ ^self ] ].
    file := File pathNameReadOnly: file.
    self class textWindowClass new openOnFile: file.
    file close! !
SpeedReadingWindow comment: ''!


!SpeedReadingTextPane class methodsFor: 'constants'!
  
fastScrollPeriod
Transcript cr; show: 'Fast scroll'.
    ^100!

slowScrollPeriod
Transcript cr; show: 'Slow scroll'.
    ^400! !

!SpeedReadingTextPane class methodsFor: 'querying'!
   
periodForSpeed: speed fontSize: pixels
| v |
Transcript cr; show: 'For speed '; print: speed; show: ' period ', (v := (((self periodForSpeed: speed) / pixels) rounded)) printString.
true ifTrue: [^v].
    ^((self periodForSpeed: speed) / pixels) rounded!
  
periodForSpeed: speed
    speed abs > 0.02 ifTrue: [^self fastScrollPeriod].
    speed abs > 0.01 ifTrue: [^self slowScrollPeriod].
    ^0! !


!SpeedReadingTextPane methodsFor: 'initializing'!
 
initialize
    super initialize.
    speedMouse := SpeedMouse new
        significantDeltaPosition: 10;
        significantDeltaTime: 40.
    autoScrolling := false! !

!SpeedReadingTextPane methodsFor: 'get/set'!

speedMouse
    ^speedMouse!

speedMouse: aSpeedMouse
    speedMouse := aSpeedMouse! !

!SpeedReadingTextPane methodsFor: 'querying'!
  
speed
    ^self speedMouse oldVelocity asPoint y! !

!SpeedReadingTextPane methodsFor: 'opening/closing'!

close
    self deactivateAutoScroll.
    ^super close! !

!SpeedReadingTextPane methodsFor: 'overrides'!

wmVScroll: wordInteger with: longInteger
    "Private - Process the vertical scroll message."
    | type position period |
    type := wordInteger lowWord. position := wordInteger highWord.
    self autoScrolling ifTrue: [self deactivateAutoScroll. ^nil].
    type = SbThumbposition ifTrue: ["end of tracking"
         speedMouse positionNow: position"; note: 'Thumb position: '"].
    type = SbThumbtrack ifTrue: [
        speedMouse positionNow: position"; note: 'Thumb track: '"].
    type = SbEndscroll ifTrue: [
        period := self class
            periodForSpeed: self speed
            fontSize: self font height.
        period > 0 ifTrue: [
            self speedMouse pushVelocity; velocity: 0.0.
            self activateAutoScroll: period]].
    ^super wmVScroll: wordInteger with: longInteger
! !

!SpeedReadingTextPane methodsFor: 'auto scrolling'!
   
autoScrolling
    ^timer notNil!
   
autoScroll
    | speed pen |
    self autoScrolling ifFalse: [^self].
    pixelsScrolled := pixelsScrolled + self speed sign negated.
    pixelsScrolled abs = self font height
        ifTrue: [
            topCorner := topCorner + (0@pixelsScrolled sign negated).
            self display; updateVerticalSlider.
            (topCorner y <= 1 or: [topCorner y = textHolder extent y]) ifTrue: [
                self deactivateAutoScroll].
            pixelsScrolled := 0]
        ifFalse: [
            pen := self pen.
            pen selfCopyToX: 0 Y: pixelsScrolled sign.
            pixelsScrolled positive
                ifTrue: [pen blank: (0@0 extent: pen width@1)]
                ifFalse: [pen blank: (0@pen height - 1 extent: pen width@1)]]
!

scrollAmountForSpeed: speed
    ^"self amountToScrollUp * "speed sign negated!
 
deactivateAutoScroll
    timer isNil ifTrue: [^self].
    timer stop.
    timer := nil
!

activateAutoScroll: period
    "Remove old timer."
    self deactivateAutoScroll.
    "Can't go up if at the top."
    (topCorner y = 0 and: [self speed negative]) ifTrue: [^self].
    "Can't go down if at the bottom."
    (topCorner y = textHolder extent y and: [self speed positive]) ifTrue: [^self].
    "Set up new timer."
    timer := Timer new
        setName: 'auto scroll timer';
        when: #ticked send: #autoScroll to: self;
        period: period;
        start.
    "Prepare for slow scrolling."
    pixelsScrolled := 0
! !
SpeedReadingTextPane comment: ''!
